#define G_LOG_DOMAIN "ap-item"

#include "ap-item.h"

#include <glib.h>
#include <glib-object.h>

G_DEFINE_INTERFACE (APItem, ap_item, G_TYPE_OBJECT);

static void
ap_item_default_init (APItemInterface *iface)
{
    g_object_interface_install_property (
        iface,
        g_param_spec_string ("name",
                             "Name of the item",
                             "Name of the item.",
                             NULL,
                             G_PARAM_READWRITE));

    g_object_interface_install_property (
        iface,
        g_param_spec_string ("description",
                             "Description of the item",
                             "Description of the item.",
                             NULL,
                             G_PARAM_READWRITE));

    g_object_interface_install_property (
        iface,
        g_param_spec_double ("quantity",
                             "Quantity of the item",
                             "Quantity of the item.",
                             G_MINDOUBLE,
                             G_MAXDOUBLE,
                             1,
                             G_PARAM_READWRITE));

    g_object_interface_install_property (
        iface,
        g_param_spec_double ("weight",
                             "Weight of the item",
                             "Weight of the item.",
                             G_MINDOUBLE,
                             G_MAXDOUBLE,
                             1,
                             G_PARAM_READWRITE));

    g_object_interface_install_property (
        iface,
        g_param_spec_double ("worth",
                             "Worth of the item",
                             "Worth of the item.",
                             G_MINDOUBLE,
                             G_MAXDOUBLE,
                             1,
                             G_PARAM_READWRITE));
}

void
ap_item_set_name (APItem *self,
                  gchar  *name)
{
    g_return_if_fail (AP_IS_ITEM (self));
    g_return_if_fail (AP_ITEM_GET_IFACE (self)->get_name);

    AP_ITEM_GET_IFACE (self)->set_name (self, name);
}

gchar *
ap_item_get_name (APItem *self)
{
    g_return_val_if_fail (AP_IS_ITEM (self), NULL);
    g_return_val_if_fail (AP_ITEM_GET_IFACE (self)->get_name, NULL);

    return AP_ITEM_GET_IFACE (self)->get_name (self);
}

void
ap_item_set_description (APItem *self,
                         gchar  *description)
{
    g_return_if_fail (AP_IS_ITEM (self));
    g_return_if_fail (AP_ITEM_GET_IFACE (self)->get_description);

    AP_ITEM_GET_IFACE (self)->set_description (self, description);
}

gchar *
ap_item_get_description (APItem *self)
{
    g_return_val_if_fail (AP_IS_ITEM (self), NULL);
    g_return_val_if_fail (AP_ITEM_GET_IFACE (self)->get_description, NULL);

    return AP_ITEM_GET_IFACE (self)->get_description (self);
}

void
ap_item_set_quantity (APItem  *self,
                      gdouble  quantity)
{
    g_return_if_fail (AP_IS_ITEM (self));
    g_return_if_fail (AP_ITEM_GET_IFACE (self)->get_quantity);

    AP_ITEM_GET_IFACE (self)->set_quantity (self, quantity);
}

gdouble
ap_item_get_quantity (APItem *self)
{
    g_return_val_if_fail (AP_IS_ITEM (self), 0);
    g_return_val_if_fail (AP_ITEM_GET_IFACE (self)->get_quantity, 0);

    return AP_ITEM_GET_IFACE (self)->get_quantity (self);
}

void
ap_item_set_weight (APItem  *self,
                    gdouble  weight)
{
    g_return_if_fail (AP_IS_ITEM (self));
    g_return_if_fail (AP_ITEM_GET_IFACE (self)->get_weight);

    AP_ITEM_GET_IFACE (self)->set_weight (self, weight);
}

gdouble
ap_item_get_weight (APItem *self)
{
    g_return_val_if_fail (AP_IS_ITEM (self), 0);
    g_return_val_if_fail (AP_ITEM_GET_IFACE (self)->get_weight, 0);

    return AP_ITEM_GET_IFACE (self)->get_weight (self);
}

void
ap_item_set_worth (APItem  *self,
                   gdouble  worth)
{
    g_return_if_fail (AP_IS_ITEM (self));
    g_return_if_fail (AP_ITEM_GET_IFACE (self)->get_worth);

    AP_ITEM_GET_IFACE (self)->set_worth (self, worth);
}

gdouble
ap_item_get_worth (APItem *self)
{
    g_return_val_if_fail (AP_IS_ITEM (self), 0);
    g_return_val_if_fail (AP_ITEM_GET_IFACE (self)->get_worth, 0);

    return AP_ITEM_GET_IFACE (self)->get_worth (self);
}
